// pecco -- please enjoy classification with conjunctive features
//  $Id: typedef.cc 699 2011-12-04 20:29:14Z ynaga $
// Copyright (c) 2008-2011 Naoki Yoshinaga <ynaga@tkl.iis.u-tokyo.ac.jp>
#include "typedef.h"

namespace ny {
  // (a slightly) secure conversion from string to numerical
  template <> fl_t  strton <fl_t>  (const char * s, char ** error)
#ifdef USE_FLOAT
  { return std::strtof (s, error); }
#else
  { return std::strtod (s, error); }
#endif
  template <> uint  strton <uint>  (const char * s, char ** error) {
    uint64_t ret = std::strtoul (s, error, 10);
    if (ret > UINT_MAX)
      print_err (HERE "overflow: %s\n", s);
    return static_cast <uint> (ret);
  }
  template <> int   strton <int>   (const char * s, char ** error) {
    int64_t ret = std::strtol (s, error, 10);
    if (ret > INT_MAX)      print_err (HERE "overflow: %s\n", s);
    else if (ret < INT_MIN) print_err (HERE "underflow: %s\n", s);
    return static_cast <int> (ret);
  }
  template <> long  strton <long>  (const char * s, char ** error)
  { return std::strtol (s, error, 10); }
  template <typename T> T strton (const char * s) {
    char* err;
    T n = strton <T> (s, &err);
    if (*err != '\0') print_err (HERE "invalid conversion: %s\n", s);
    return n;
  }
  // explict specialization
  template fl_t strton (const char *);
  template uint strton (const char *);
  template int  strton (const char *);
}
